package com.yanzuoguang.dao;


import com.yanzuoguang.util.vo.PageSizeData;
import com.yanzuoguang.util.vo.PageSizeReqVo;

import java.util.Collection;
import java.util.List;

/**
 * 数据基本操作接口
 *
 * @author 颜佐光
 */
public interface BaseDao {

    /**
     * 创建数据,当不传入了主键时，则会自动生成主键，传入时不会生成。
     *
     * @param model 需要创建的数据
     * @return 创建的主键编号
     */
    String create(Object model);

    /**
     * 修改数据
     *
     * @param model 需要修改的数据
     * @return 删除的主键编号
     */
    String update(Object model);

    /**
     * 保存数据,有主键时修改，无主键时创建
     *
     * @param model 需要保存的数据
     * @return 保存的主键编号
     */
    String save(Object model);

    /**
     * 删除数据，可以用于父子表删除，如通过订单删除游客信息 visitorDao.remove({orderId:1});
     * int field;
     *
     * @param model 需要删除的数据,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @return 删除的记录数量
     */
    int remove(Object model);

    /**
     * 加载数据，可以用于父子表删除，如通过订单删除游客信息 visitorDao.load({orderId:1});
     *
     * @param model       加载数据的请求参数,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @param resultClass 需要加载的数据的类型
     * @param <T>         返回数据的类型
     * @return 需要返回的数据
     */
    <T extends Object> T load(Object model, Class<T> resultClass);


    /**
     * 加载列表数据，可以用于父子表删除，如通过订单删除游客信息 visitorDao.loadList({orderId:1});
     *
     * @param model       加载数据的请求参数,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @param resultClass 需要加载的数据的类型
     * @param <T>         返回数据的类型
     * @return 需要返回的数据
     */
    <T extends Object> List<T> loadList(Object model, Class<T> resultClass);

    /**
     * 加载分页数据
     *
     * @param model       加载数据的请求参数,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @param resultClass 需要加载的数据的类型
     * @param <T>         返回数据的类型
     * @return 需要返回的数据
     */
    <T extends Object> PageSizeData<T> loadPage(PageSizeReqVo model, Class<T> resultClass);

    /**
     * 修改数据
     *
     * @param model 需要修改的数据
     * @return 删除的主键编号
     */
    List<String> updateList(Collection model);

    /**
     * 修改数据
     *
     * @param model 需要修改的数据
     * @return 删除的主键编号
     */
    List<String> updateArray(Object... model);

    /**
     * 创建数据,当不传入了主键时，则会自动生成主键，传入时不会生成。
     *
     * @param model 需要创建的数据
     * @return 创建的主键编号
     */
    List<String> createList(List model);

    /**
     * 创建数据,当不传入了主键时，则会自动生成主键，传入时不会生成。
     *
     * @param model 需要创建的数据
     * @return 创建的主键编号
     */
    List<String> createArray(Object... model);

    /**
     * 保存数据,有主键时修改，无主键时创建
     *
     * @param model 需要保存的数据
     * @return 保存的主键编号
     */
    List<String> saveList(List model);

    /**
     * 保存数据,有主键时修改，无主键时创建
     *
     * @param model 需要保存的数据
     * @return 保存的主键编号
     */
    List<String> saveArray(Object... model);

    /**
     * 删除数据，可以用于父子表删除，如通过订单删除游客信息 visitorDao.remove({orderId:1});
     * int field;
     *
     * @param model 需要删除的数据,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @return 删除的记录数量
     */
    int removeList(List model);

    /**
     * 删除数据，可以用于父子表删除，如通过订单删除游客信息 visitorDao.remove({orderId:1});
     * int field;
     *
     * @param model 需要删除的数据,可以是主键字符串（Int），或者是包含主键的实体，或者是包含其他非主键的实体完全匹配.
     * @return 删除的记录数量
     */
    int removeArray(Object... model);
}
